@@ -5,49 +5,38 @@
 
 from __future__ import annotations
 
+import contextlib
 import functools
 import os
 import sys
 
+from collections.abc import Mapping
 from types import FrameType
-from typing import (
-    cast, Any, Callable, Dict, List, Mapping, Optional, Set, Type, TypeVar,
-)
+from typing import cast, Any, Callable, TypeVar
 
 from coverage import env
 from coverage.config import CoverageConfig
+from coverage.core import Core
 from coverage.data import CoverageData
 from coverage.debug import short_stack
-from coverage.disposition import FileDisposition
 from coverage.exceptions import ConfigError
 from coverage.misc import human_sorted_items, isolate_module
 from coverage.plugin import CoveragePlugin
-from coverage.pytracer import PyTracer
-from coverage.sysmon import SysMonitor
 from coverage.types import (
-    TArc, TFileDisposition, TTraceData, TTraceFn, TracerCore, TWarnFn,
+    TArc,
+    TCheckIncludeFn,
+    TFileDisposition,
+    TShouldStartContextFn,
+    TShouldTraceFn,
+    TTraceData,
+    TTraceFn,
+    Tracer,
+    TWarnFn,
 )
 
 os = isolate_module(os)
 
 
-try:
-    # Use the C extension code when we can, for speed.
-    from coverage.tracer import CTracer, CFileDisposition
-    HAS_CTRACER = True
-except ImportError:
-    # Couldn't import the C extension, maybe it isn't built.
-    if os.getenv("COVERAGE_CORE") == "ctrace":      # pragma: part covered
-        # During testing, we use the COVERAGE_CORE environment variable
-        # to indicate that we've fiddled with the environment to test this
-        # fallback code.  If we thought we had a C tracer, but couldn't import
-        # it, then exit quickly and clearly instead of dribbling confusing
-        # errors. I'm using sys.exit here instead of an exception because an
-        # exception here causes all sorts of other noise in unittest.
-        sys.stderr.write("*** COVERAGE_CORE is 'ctrace' but can't import CTracer!\n")
-        sys.exit(1)
-    HAS_CTRACER = False
-
 T = TypeVar("T")
 
 
@@ -70,22 +59,21 @@ class Collector:
     # The stack of active Collectors.  Collectors are added here when started,
     # and popped when stopped.  Collectors on the stack are paused when not
     # the top, and resumed when they become the top again.
-    _collectors: List[Collector] = []
+    _collectors: list[Collector] = []
 
     # The concurrency settings we support here.
     LIGHT_THREADS = {"greenlet", "eventlet", "gevent"}
 
     def __init__(
         self,
-        should_trace: Callable[[str, FrameType], TFileDisposition],
-        check_include: Callable[[str, FrameType], bool],
-        should_start_context: Optional[Callable[[FrameType], Optional[str]]],
+        core: Core,
+        should_trace: TShouldTraceFn,
+        check_include: TCheckIncludeFn,
+        should_start_context: TShouldStartContextFn | None,
         file_mapper: Callable[[str], str],
-        timid: bool,
         branch: bool,
         warn: TWarnFn,
-        concurrency: List[str],
-        metacov: bool,
+        concurrency: list[str],
     ) -> None:
         """Create a collector.
 
@@ -104,11 +92,6 @@ class Collector:
         filename.  The result is the name that will be recorded in the data
         file.
 
-        If `timid` is true, then a slower simpler trace function will be
-        used.  This is important for some environments where manipulation of
-        tracing functions make the faster more sophisticated trace function not
-        operate properly.
-
         If `branch` is true, then branches will be measured.  This involves
         collecting data on which statements followed each other (arcs).  Use
         `get_arc_data` to get the arc data.
@@ -123,6 +106,7 @@ class Collector:
         Other values are ignored.
 
         """
+        self.core = core
         self.should_trace = should_trace
         self.check_include = check_include
         self.should_start_context = should_start_context
@@ -136,53 +120,12 @@ class Collector:
 
         self.covdata: CoverageData
         self.threading = None
-        self.static_context: Optional[str] = None
+        self.static_context: str | None = None
 
         self.origin = short_stack()
 
         self.concur_id_func = None
 
-        self._trace_class: Type[TracerCore]
-        self.file_disposition_class: Type[TFileDisposition]
-
-        core: Optional[str]
-        if timid:
-            core = "pytrace"
-        else:
-            core = os.getenv("COVERAGE_CORE")
-            if not core:
-                # Once we're comfortable with sysmon as a default:
-                # if env.PYBEHAVIOR.pep669 and self.should_start_context is None:
-                #     core = "sysmon"
-                if HAS_CTRACER:
-                    core = "ctrace"
-                else:
-                    core = "pytrace"
-
-        if core == "sysmon":
-            self._trace_class = SysMonitor
-            self._core_kwargs = {"tool_id": 3 if metacov else 1}
-            self.file_disposition_class = FileDisposition
-            self.supports_plugins = False
-            self.packed_arcs = False
-            self.systrace = False
-        elif core == "ctrace":
-            self._trace_class = CTracer
-            self._core_kwargs = {}
-            self.file_disposition_class = CFileDisposition
-            self.supports_plugins = True
-            self.packed_arcs = True
-            self.systrace = True
-        elif core == "pytrace":
-            self._trace_class = PyTracer
-            self._core_kwargs = {}
-            self.file_disposition_class = FileDisposition
-            self.supports_plugins = False
-            self.packed_arcs = False
-            self.systrace = True
-        else:
-            raise ConfigError(f"Unknown core value: {core!r}")
-
         # We can handle a few concurrency options here, but only one at a time.
         concurrencies = set(self.concurrency)
         unknown = concurrencies - CoverageConfig.CONCURRENCY_CHOICES
@@ -216,11 +159,11 @@ class Collector:
             msg = f"Couldn't trace with concurrency={tried}, the module isn't installed."
             raise ConfigError(msg) from ex
 
-        if self.concur_id_func and not hasattr(self._trace_class, "concur_id_func"):
+        if self.concur_id_func and not hasattr(core.tracer_class, "concur_id_func"):
             raise ConfigError(
                 "Can't support concurrency={} with {}, only threads are supported.".format(
                     tried, self.tracer_name(),
-                )
+                ),
             )
 
         if do_threading or not concurrencies:
@@ -235,7 +178,7 @@ class Collector:
     def __repr__(self) -> str:
         return f"<Collector at {id(self):#x}: {self.tracer_name()}>"
 
-    def use_data(self, covdata: CoverageData, context: Optional[str]) -> None:
+    def use_data(self, covdata: CoverageData, context: str | None) -> None:
         """Use `covdata` for recording data."""
         self.covdata = covdata
         self.static_context = context
@@ -243,29 +186,32 @@ class Collector:
 
     def tracer_name(self) -> str:
         """Return the class name of the tracer we're using."""
-        return self._trace_class.__name__
+        return self.core.tracer_class.__name__
 
     def _clear_data(self) -> None:
         """Clear out existing data, but stay ready for more collection."""
         # We used to use self.data.clear(), but that would remove filename
         # keys and data values that were still in use higher up the stack
         # when we are called as part of switch_context.
-        for d in self.data.values():
-            d.clear()
+        with self.data_lock or contextlib.nullcontext():
+            for d in self.data.values():
+                d.clear()
 
         for tracer in self.tracers:
             tracer.reset_activity()
 
     def reset(self) -> None:
         """Clear collected data, and prepare to collect more."""
+        self.data_lock = self.threading.Lock() if self.threading else None
+
         # The trace data we are collecting.
         self.data: TTraceData = {}
 
         # A dictionary mapping file names to file tracer plugin names that will
         # handle them.
-        self.file_tracers: Dict[str, str] = {}
+        self.file_tracers: dict[str, str] = {}
 
-        self.disabled_plugins: Set[str] = set()
+        self.disabled_plugins: set[str] = set()
 
         # The .should_trace_cache attribute is a cache from file names to
         # coverage.FileDisposition objects, or None.  When a file is first
@@ -296,14 +242,26 @@ class Collector:
             self.should_trace_cache = {}
 
         # Our active Tracers.
-        self.tracers: List[TracerCore] = []
+        self.tracers: list[Tracer] = []
 
         self._clear_data()
 
+    def lock_data(self) -> None:
+        """Lock self.data_lock, for use by the C tracer."""
+        if self.data_lock is not None:
+            self.data_lock.acquire()
+
+    def unlock_data(self) -> None:
+        """Unlock self.data_lock, for use by the C tracer."""
+        if self.data_lock is not None:
+            self.data_lock.release()
+
     def _start_tracer(self) -> TTraceFn | None:
         """Start a new Tracer object, and store it in self.tracers."""
-        tracer = self._trace_class(**self._core_kwargs)
+        tracer = self.core.tracer_class(**self.core.tracer_kwargs)
         tracer.data = self.data
+        tracer.lock_data = self.lock_data
+        tracer.unlock_data = self.unlock_data
         tracer.trace_arcs = self.branch
         tracer.should_trace = self.should_trace
         tracer.should_trace_cache = self.should_trace_cache
@@ -337,12 +295,12 @@ class Collector:
     #
     # New in 3.12: threading.settrace_all_threads: https://github.com/python/cpython/pull/96681
 
-    def _installation_trace(self, frame: FrameType, event: str, arg: Any) -> Optional[TTraceFn]:
+    def _installation_trace(self, frame: FrameType, event: str, arg: Any) -> TTraceFn | None:
         """Called on new threads, installs the real tracer."""
         # Remove ourselves as the trace function.
         sys.settrace(None)
         # Install the real tracer.
-        fn: Optional[TTraceFn] = self._start_tracer()
+        fn: TTraceFn | None = self._start_tracer()
         # Invoke the real trace function with the current event, to be sure
         # not to lose an event.
         if fn:
@@ -382,7 +340,7 @@ class Collector:
 
         # Install our installation tracer in threading, to jump-start other
         # threads.
-        if self.systrace and self.threading:
+        if self.core.systrace and self.threading:
             self.threading.settrace(self._installation_trace)
 
     def stop(self) -> None:
@@ -419,7 +377,7 @@ class Collector:
         """Resume tracing after a `pause`."""
         for tracer in self.tracers:
             tracer.start()
-        if self.systrace:
+        if self.core.systrace:
             if self.threading:
                 self.threading.settrace(self._installation_trace)
             else:
@@ -439,9 +397,9 @@ class Collector:
         """
         return any(tracer.activity() for tracer in self.tracers)
 
-    def switch_context(self, new_context: Optional[str]) -> None:
+    def switch_context(self, new_context: str | None) -> None:
         """Switch to a new dynamic context."""
-        context: Optional[str]
+        context: str | None
         self.flush_data()
         if self.static_context:
             context = self.static_context
@@ -461,12 +419,12 @@ class Collector:
         plugin._coverage_enabled = False
         disposition.trace = False
 
-    @functools.lru_cache(maxsize=None)          # pylint: disable=method-cache-max-size-none
+    @functools.cache          # pylint: disable=method-cache-max-size-none
     def cached_mapped_file(self, filename: str) -> str:
         """A locally cached version of file names mapped through file_mapper."""
         return self.file_mapper(filename)
 
-    def mapped_file_dict(self, d: Mapping[str, T]) -> Dict[str, T]:
+    def mapped_file_dict(self, d: Mapping[str, T]) -> dict[str, T]:
         """Return a dict like d, but with keys modified by file_mapper."""
         # The call to list(items()) ensures that the GIL protects the dictionary
         # iterator against concurrent modifications by tracers running
@@ -502,15 +460,18 @@ class Collector:
             return False
 
         if self.branch:
-            if self.packed_arcs:
+            if self.core.packed_arcs:
                 # Unpack the line number pairs packed into integers.  See
                 # tracer.c:CTracer_record_pair for the C code that creates
                 # these packed ints.
-                arc_data: Dict[str, List[TArc]] = {}
-                packed_data = cast(Dict[str, Set[int]], self.data)
-                for fname, packeds in packed_data.items():
+                arc_data: dict[str, list[TArc]] = {}
+                packed_data = cast(dict[str, set[int]], self.data)
+
+                # The list() here and in the inner loop are to get a clean copy
+                # even as tracers are continuing to add data.
+                for fname, packeds in list(packed_data.items()):
                     tuples = []
-                    for packed in packeds:
+                    for packed in list(packeds):
                         l1 = packed & 0xFFFFF
                         l2 = (packed & (0xFFFFF << 20)) >> 20
                         if packed & (1 << 40):
@@ -520,10 +481,10 @@ class Collector:
                         tuples.append((l1, l2))
                     arc_data[fname] = tuples
             else:
-                arc_data = cast(Dict[str, List[TArc]], self.data)
+                arc_data = cast(dict[str, list[TArc]], self.data)
             self.covdata.add_arcs(self.mapped_file_dict(arc_data))
         else:
-            line_data = cast(Dict[str, Set[int]], self.data)
+            line_data = cast(dict[str, set[int]], self.data)
             self.covdata.add_lines(self.mapped_file_dict(line_data))
 
         file_tracers = {
