@@ -13,8 +13,9 @@ import os.path
 import re
 
 from typing import (
-    Any, Callable, Iterable, Union,
+    Any, Callable, Union,
 )
+from collections.abc import Iterable
 
 from coverage.exceptions import ConfigError
 from coverage.misc import isolate_module, human_sorted_items, substitute_variables
@@ -250,6 +251,7 @@ class CoverageConfig(TConfigurable, TPluginConfig):
 
         # Defaults for [lcov]
         self.lcov_output = "coverage.lcov"
+        self.lcov_line_checksums = False
 
         # Defaults for [paths]
         self.paths: dict[str, list[str]] = {}
@@ -428,6 +430,7 @@ class CoverageConfig(TConfigurable, TPluginConfig):
 
         # [lcov]
         ("lcov_output", "lcov:output"),
+        ("lcov_line_checksums", "lcov:line_checksums", "boolean")
     ]
 
     def _set_attr_from_config_option(
@@ -612,6 +615,11 @@ def read_coverage_config(
     # 4) from constructor arguments:
     config.from_args(**kwargs)
 
+    # 5) for our benchmark, force settings using a secret environment variable:
+    force_file = os.getenv("COVERAGE_FORCE_CONFIG")
+    if force_file:
+        config.from_file(force_file, warn, our_file=True)
+
     # Once all the config has been collected, there's a little post-processing
     # to do.
     config.post_process()
